#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import datareader
import config

import itertools
import numpy


results = datareader.load_results(config.PROJECT_NAMES, config.CATEGORIES, config.RESULT_FOLDER)
filtered_results = datareader.filter_results(results, config.PROJECT_NAMES, config.CATEGORIES)

###############################################################################
#### Descriptive statistics ###################################################
###############################################################################

def forall_projects(f, results):
    """Takes a function f and loops through every project and every type
    (atomic vs tangled). Applies f to those results."""
    for project in config.PROJECT_NAMES:
        for category in config.CATEGORIES:
            print(project, category, f(results[project][category]), sep='\t')

def commit_count(res):
    commitcount = len(set([x.sha for x in res]))
    return commitcount

def median_file_count(res):
    """Count the number of java files per commit, return the median."""
    grouped = itertools.groupby(res, lambda r: r.sha)
    ctrlist = []
    for key, group in grouped:
        ctr = len([r for r in group if r.file.endswith('java')])
        ctrlist.append(ctr)
    return numpy.median(ctrlist)

def median_nodes_found(res):
    """Useful ones: just those for which a node was found."""
    return numpy.median([r.node_found for r in res])

forall_projects(commit_count, results)
print('---')
forall_projects(commit_count, filtered_results)
print('---')
forall_projects(median_file_count, results)
print('---')
forall_projects(median_file_count, filtered_results)
print('---')
forall_projects(median_nodes_found, filtered_results)
print('---')

###############################################################################
#### Manual Filter ############################################################
###############################################################################

final_results = datareader.filter_results_with_manual(filtered_results,
                                                      config.PROJECT_NAMES,
                                                      config.CATEGORIES)

print('After manual filtering')
forall_projects(commit_count, final_results)
print('---')
forall_projects(median_file_count, final_results)
print('---')
forall_projects(median_nodes_found, final_results)
print('---')