#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""We want to parse the logs to find out running times of everything."""

from datetime import datetime

import config

def parse_logs():
    """Open every log file, parse out times for each commit."""
    results = {}
    for project in config.PROJECT_NAMES:
        results[project] = {}
        for category in config.CATEGORIES:
            results[project][category] = {}
            try:
                path = config.RESULT_FOLDER + project + '.' + category + '.log'
                with open(path) as f:
                    results[project][category].update(parse_file(f))
            except FileNotFoundError:
                # Do something about those where there are two logs
                for i in [1,2]:
                    path = config.RESULT_FOLDER + project + '.' + category + '.' + str(i) + '.log'
                    with open(path) as f:
                        results[project][category].update(parse_file(f))
    return results

def parse_file(f):
    """Parses a log file for begin and end of commit analysis."""
    result = {}
    last_sha = None
    for line in f:
        line = line.strip()
        if line.endswith('handleCommit'):
            # Parse out start time
            starttime = parse_time(line)

            nextline = next(f).strip()
            if nextline.startswith('INFO: Handling commit:'):
                # Drop the info etc part
                sha = nextline[23:]
                result[sha] = { 'starttime': starttime }
                
                if last_sha != None:
                    result[last_sha]['endtime'] = starttime
                last_sha = sha
        if line.endswith('main'):
            endtime = parse_time(line)
            nextline = next(f).strip()
            if nextline == 'INFO: Done':
                result[last_sha]['endtime'] = endtime
    return result

def parse_time(line):
    """Given a line of the log starting with the time, parse out that time."""
    pos = 0
    am_pos = line.find('AM')
    pm_pos = line.find('PM')
    if am_pos == -1 and pm_pos == -1:
        raise Exception
    elif am_pos == -1:
        pos = pm_pos
    elif pm_pos == -1:
        pos = am_pos
    else:
        pos = min(am_pos, pm_pos)
    datestr = line[0:pos+2]

    # !!! Will fail in different locale due to %b and %p !!!
    t = datetime.strptime(datestr, '%b %d, %Y %I:%M:%S %p')
    return t

if __name__ == '__main__':
    r = parse_logs()
    for project in r:
        for category in r[project]:
            print(len(r[project][category]))