#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import datareader
import config

import itertools

import matplotlib.pyplot as plt

results = datareader.load_results(config.PROJECT_NAMES, config.CATEGORIES, config.RESULT_FOLDER)
filtered_results = datareader.filter_results(results, config.PROJECT_NAMES, config.CATEGORIES)
final_results = filtered_results
#final_results = datareader.filter_results_with_manual(filtered_results,
#                                                      config.PROJECT_NAMES,
#                                                      config.CATEGORIES)

###############################################################################
#### RQ1 ######################################################################
###############################################################################

def identification_numbers(res):
    """For the given project/category selection, return the total number of SHAs
    and the number of those that had a clustercount of 1."""
    grouped = itertools.groupby(res, lambda r: r.sha)
    onecount = 0
    total = 0
    for sha, group in grouped:
        total = total + 1
        
        # Find highest clustercount
        clustercount = max([result.clusters for result in group])

        if clustercount == 1:
            onecount = onecount + 1
    return [total, onecount]

def rq1_numbers(results):
    print('project',
          'total',
          '#t+',
          '#t-',
          '#f+',
          '#f-',
          'precision',
          'recall',
          'fscore',
          sep='\t')
    endresult = []
    for project in config.PROJECT_NAMES:
        atomic_analysis = identification_numbers(results[project]['atomic'])
        tangled_analysis = identification_numbers(results[project]['tangled'])
        total = atomic_analysis[0] + tangled_analysis[0]
        true_positive = tangled_analysis[0] - tangled_analysis[1]
        true_negative = atomic_analysis[1]
        false_positive = atomic_analysis[0] - atomic_analysis[1]
        false_negative = tangled_analysis[1]
        precision = true_positive / (true_positive + false_positive)
        recall = true_positive / (true_positive + false_negative)
        fscore = 2 / (1 / recall + 1 / precision)
        print(project,
              total,
              true_positive,
              true_negative,
              false_positive,
              false_negative,
              precision,
              recall,
              fscore,
              sep='\t')
        print(project,
              total / total * 100,
              round(true_positive / total * 100),
              round(true_negative / total * 100),
              round(false_positive / total * 100),
              round(false_negative / total * 100),
              precision,
              recall,
              fscore,
              sep='\t')
        endresult.append([
                project,
              total,
              true_positive,
              true_negative,
              false_positive,
              false_negative,
              precision,
              recall,
              fscore
                ])
    return endresult
        
r = rq1_numbers(final_results)

# Make a pie chart
def make_piechart(res, idx):
    colors = ['#fef0d9', '#fdcc8a', '#fc8d59', '#d7301f']
    labels = ['TP', 'TN', 'FP', 'FN']
    plt.subplot(2, 3, idx)
    sizes = [res[2], res[3], res[4], res[5]]
    sizes = [x / res[1] * 100 for x in sizes]
    plt.pie(sizes, labels=labels, colors=colors)
    plt.axis('equal')
    #plt.show()

f = plt.figure()
for idx in range(5):
    make_piechart(r[idx], idx+1)
plt.show()
f.savefig('rq1-piechart.pdf', bbox_inches='tight')


## USE r FOR RQ2.
print()
print('RQ2 things')
print('----------')
def atomic_f_score(res):
    precision = res[3] / (res[3] + res[5])
    recall = res[3] / (res[3] + res[4])
    fscore = 2 / (1 / recall + 1 / precision)
    return (precision, recall, fscore)
for res in r:
    scores = list(map(round, atomic_f_score(res), itertools.repeat(2)))
    print(res[0], scores[0], scores[1], scores[2], sep='\t')
