#!/usr/bin/env python3
# -*- coding: utf-8 -*-


import datareader
import config

import itertools

from matplotlib import rcParams
rcParams['font.family'] = 'serif'
rcParams['font.serif'] = ['Linux Libertine Display O']

import matplotlib.pyplot as plt
import scipy.stats as stats

results = datareader.load_results(config.PROJECT_NAMES, config.CATEGORIES, config.RESULT_FOLDER)
filtered_results = datareader.filter_results(results, config.PROJECT_NAMES, config.CATEGORIES)
final_results = filtered_results # It is already filtered.
#final_results = datareader.filter_results_with_manual(filtered_results,
#                                                      config.PROJECT_NAMES,
#                                                      config.CATEGORIES)

blobfolder = '/Users/ward/phd/slicing/slicing-data/categories/original/'
blob_info = datareader.load_blob_info(config.PROJECT_NAMES, blobfolder)


def correct_partition_number(results, dataset):
    clusterresults = {}
    print('project', 'commits', 'correct', sep='\t')
    for project in config.PROJECT_NAMES:
        clusterresults[project] = {}
        
        total = 0
        correct = 0
        
        grouped = itertools.groupby(results[project]['tangled'], lambda r: r.sha)
        for sha, group in grouped:
            if sha in dataset[project]:
                clustercount = max([result.clusters for result in group])
                
                clusterresults[project][sha] = (clustercount, dataset[project][sha])
                
                total = total + 1
                if dataset[project][sha] == clustercount:
                    correct = correct + 1
        print(project, total, correct, sep='\t')
    return clusterresults



#cluster_comparison = correct_partition_number(filtered_results, blob_info)
cluster_comparison = correct_partition_number(final_results, blob_info)    

for project in cluster_comparison:
    if len(cluster_comparison[project]) == 0:
        continue
    real = [cluster_comparison[project][sha][0] for sha in cluster_comparison[project]]
    real_correct = [cluster_comparison[project][sha][0] \
                    for sha in cluster_comparison[project] \
                    if cluster_comparison[project][sha][0] == cluster_comparison[project][sha][1]]
    real_wrong = [cluster_comparison[project][sha][0] \
                    for sha in cluster_comparison[project] \
                    if cluster_comparison[project][sha][0] != cluster_comparison[project][sha][1]]
    dataset = [cluster_comparison[project][sha][1] for sha in cluster_comparison[project]]
    print(project, stats.wilcoxon(real, dataset))
    bins = range(15)
    xticks = range(15)
    f = None
    if project == 'jruby':
        f = plt.figure()
    plt.hist([real, dataset], bins=bins)
    plt.legend(('Technique', 'Dataset'))
    plt.show()
    if project == 'jruby':
        f.savefig('jruby_partition_comparison.pdf', bbox_inches='tight')
    
    plt.hist([real_correct, real_wrong, dataset], bins=bins)
    plt.legend(['Correct', 'Wrong', 'Dataset'])
    plt.xticks(xticks)
    plt.show()
        
    high_real = [x for x in real if x > 15]
    print(high_real)
