#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Get times of analysis."""

import config
import logreader
import datareader

times = logreader.parse_logs()

results = datareader.load_results(config.PROJECT_NAMES, config.CATEGORIES, config.RESULT_FOLDER)
filtered_results = datareader.filter_results(results, config.PROJECT_NAMES, config.CATEGORIES)
final_results = datareader.filter_results_with_manual(filtered_results,
                                                      config.PROJECT_NAMES,
                                                      config.CATEGORIES)

def filter_times(times, final_results):
    """Keeps only the times that matter. Places them in updated category.
    Adds a timediff to the object."""
    new_times = {}
    for project in times:
        new_times[project] = {}
        new_times[project]['atomic'] = {}
        new_times[project]['tangled'] = {}
        for category in times[project]:
            # Get SHA from both, some may have switched category after our manual
            # analysis of the commits.
            validatomicshas = list(map(lambda r: r.sha, final_results[project]['atomic']))
            validtangledshas = list(map(lambda r: r.sha, final_results[project]['tangled']))
            #print(validshas)
            for sha in times[project][category]:
                r = times[project][category][sha]
                #print(r)
                if 'starttime' in r and 'endtime' in r:
                    if sha in validatomicshas:
                        r['diff'] = r['endtime'] - r['starttime']
                        new_times[project]['atomic'][sha] = r
                    elif sha in validtangledshas:
                        r['diff'] = r['endtime'] - r['starttime']
                        new_times[project]['tangled'][sha] = r
                    else:
                        continue
                        print('Did not find', project, category, sha)
                else:
                    continue
                    print('Time missing for', project, category, sha)
    return new_times

times = filter_times(times, final_results)

import matplotlib.pyplot as plt
import numpy as np

all_times = []
morethanminute = []
for project in times:
    #for category in times[project]:
    t1 = [times[project]['atomic'][sha]['diff'].total_seconds() for sha in times[project]['atomic']]
    t2 = [times[project]['tangled'][sha]['diff'].total_seconds() for sha in times[project]['tangled']]
    t = t1 + t2
    morethanminute += [x for x in t if x > 59]
    all_times.append(t)
    print(project, np.percentile(t, 25), np.percentile(t, 50), np.percentile(t, 75), sep='\t')
plt.boxplot(all_times)
axes = plt.gca()
axes.set_ylim([180, 1500])
